function [x,n] = aitkenPlot(fun,dfun,x,tolx,maxiter)

%function [x,n] = aitken(fun,dfun,x,tolx,maxiter)
%
%cerca uno zero della funzione 'f' col metodo di accelerazione
%di Aitken
%
%fun e' la stringa della funzione
%dfun e' la stringa della derivata di f
%x_0 e' il punto di partenza del metodo
%tolx e' il valore della tolleranza della variabile indipendente (>0)
%maxiter e' il numero massimo di iterazioni consentite
%
%I valori di ritorno:
%x valore dello zero della funzione
%n numero di iterazioni eseguite
%
%ESEMPIO: newton('sin(x)','cos(x)',1,eps,100)

if tolx <= 0
    error('tolx deve avere un valore positivo')
end


f=inline(fun);
df=inline(dfun);
n=0;

%disegno
minx=x;
maxx=x;
miny=0;
maxy=0;

clf
hold on;
fx=f(x);
plot([x x],[0 fx], 'r');


fx=f(x);
dfx=df(x);
toly=abs(tolx*dfx);

while n<maxiter && abs(fx)>toly
  x1=x-fx/dfx;
  fx1=f(x1);
  dfx1=df(x1);

  
  if x1>maxx
    maxx=x1;
  end
  if x1 <minx
    minx=x1;
  end
  
  
  x2=x1-fx1/dfx1;

  
  if x2>maxx
    maxx=x2;
  end
  if x2 <minx
    minx=x2;
  end
  
  
      plot([x1 x1],[0 fx1], 'r');  
  plot([x1 x],[0 fx], ':r');
  
  x=(x1^2-x*x2)/(2*x1-x-x2);
  
  
  fx=f(x);
  dfx=df(x);
  fx2=f(x2);

  
  if x>maxx
    maxx=x;
  end
  if x <minx
    minx=x;
  end
  
  
  toly=abs(tolx*dfx);
  n=n+3;
   

  

  
    plot([x2 x2],[0 fx2], 'r');  
  plot([x2 x1],[0 fx1], ':r');
  
    plot([x x],[0 fx], 'g');  
  plot([x x2],[0 fx2], ':g'); 
  
  
end


diffx=maxx-minx;
diffy=maxy-miny;

Xs=minx:diffx/2000:maxx;
Ys=feval(f,Xs);


plot([minx maxx],[0 0],'k');
plot(Xs,Ys,'b');
plot(x,0,'sr');