function p = TrustRegionCauchy(p)
%trust region Newton e Cauchy point

tol=getTol(p);
tol2=tol^2;
maxiter=getMaxiter(p);

maxDelta=.5;
delta=.2;
eta=1/5;

x=getStart(p); %Prendo il punto iniziale
fx=getValue(p,x);% valuto la funzione nel punto
gx=getGrad(p,x);% calcolo il gradiente nel punto
hx=getHess(p,x);% calcolo l' hessiana nel punto

n=0;

while n<maxiter && gx'*gx>tol2
    %--- trovo il punto di cauchy (cp)
    d2=gx'*hx*gx;
    gradnorm=norm(gx);
    if d2 > 0
        tau=min([1/gradnorm gradnorm^2/delta/d2]);
    else
        tau=1/gradnorm;
    end
    
    %--- ottimizzata con 1/gradnorm
    cp=-tau*delta*gx;
    %cp=-tau*delta*gk/gradnorm;
    
    fxkpk=getValue(p,x+cp);
    
    %--- aggiorno la trust region
    mkpk=fx+gx'*cp+cp'*hx*cp/2;
    rhok=(fx-fxkpk)/(fx-mkpk);
    
    if rhok < 1/4
        delta=norm(cp)/4;
    elseif rhok > 3/4 && norm(cp)==delta
        delta=min(2*delta,maxDelta);
    end
    
    %--- se 
    if rhok > eta
        x=x+cp;
        fx=fxkpk;
        nextPoint(p,x);
        n=n+1;
        gx=getGrad(p,x); %riaggiorno il gradiente
        hx=getHess(p,x); %riaggiorno l'hessiana
    end
end